# coding: utf-8

import pytest   # NOQA
import ruamel.yaml

from roundtrip import round_trip, dedent, round_trip_load, round_trip_dump  # NOQA


class Monster(ruamel.yaml.YAMLObject):
    yaml_tag = u'!Monster'

    def __init__(self, name, hp, ac, attacks):
        self.name = name
        self.hp = hp
        self.ac = ac
        self.attacks = attacks

    def __repr__(self):
        return "%s(name=%r, hp=%r, ac=%r, attacks=%r)" % (
            self.__class__.__name__, self.name, self.hp, self.ac, self.attacks)


def test_monster():
    data = ruamel.yaml.load(dedent("""\
    --- !Monster
    name: Cave spider
    hp: [2,6]    # 2d6
    ac: 16
    attacks: [BITE, HURT]
    """), Loader=ruamel.yaml.Loader)
    # normal dump, keys will be sorted
    assert ruamel.yaml.dump(data) == dedent("""\
    !Monster
    ac: 16
    attacks: [BITE, HURT]
    hp: [2, 6]
    name: Cave spider
    """)



# coding: utf-8

from __future__ import print_function

"""
testing of anchors and the aliases referring to them
"""

import sys
import textwrap
import pytest
from ruamel.yaml import YAML
from ruamel.yaml.constructor import DuplicateKeyError
from ruamel.std.pathlib import Path


class TestNewAPI:
    def test_duplicate_keys_00(self):
        from ruamel.yaml.constructor import DuplicateKeyError
        yaml = YAML()
        with pytest.raises(DuplicateKeyError):
            yaml.load('{a: 1, a: 2}')

    def test_duplicate_keys_01(self):
        yaml = YAML(typ='safe', pure=True)
        with pytest.raises(DuplicateKeyError):
            yaml.load('{a: 1, a: 2}')

    # @pytest.mark.xfail(strict=True)
    def test_duplicate_keys_02(self):
        yaml = YAML(typ='safe')
        with pytest.raises(DuplicateKeyError):
            yaml.load('{a: 1, a: 2}')

    def test_issue_135(self):
        # reported by Andrzej Ostrowski
        data = {'a': 1, 'b': 2}
        yaml = YAML(typ='safe')
        # originally on 2.7: with pytest.raises(TypeError):
        yaml.dump(data, sys.stdout)

    def test_issue_135_temporary_workaround(self):
        # never raised error
        data = {'a': 1, 'b': 2}
        yaml = YAML(typ='safe', pure=True)
        yaml.dump(data, sys.stdout)


class TestWrite:
    def test_dump_path(self, tmpdir):
        fn = Path(str(tmpdir)) / 'test.yaml'
        yaml = YAML()
        data = yaml.map()
        data['a'] = 1
        data['b'] = 2
        yaml.dump(data, fn)
        assert fn.read_text() == "a: 1\nb: 2\n"

    def test_dump_file(self, tmpdir):
        fn = Path(str(tmpdir)) / 'test.yaml'
        yaml = YAML()
        data = yaml.map()
        data['a'] = 1
        data['b'] = 2
        with open(str(fn), 'w') as fp:
            yaml.dump(data, fp)
        assert fn.read_text() == "a: 1\nb: 2\n"

    def test_dump_missing_stream(self):
        yaml = YAML()
        data = yaml.map()
        data['a'] = 1
        data['b'] = 2
        with pytest.raises(TypeError):
            yaml.dump(data)

    def test_dump_too_many_args(self, tmpdir):
        fn = Path(str(tmpdir)) / 'test.yaml'
        yaml = YAML()
        data = yaml.map()
        data['a'] = 1
        data['b'] = 2
        with pytest.raises(TypeError):
            yaml.dump(data, fn, True)

    def test_transform(self, tmpdir):
        def tr(s):
            return s.replace(' ', '  ')

        fn = Path(str(tmpdir)) / 'test.yaml'
        yaml = YAML()
        data = yaml.map()
        data['a'] = 1
        data['b'] = 2
        yaml.dump(data, fn, transform=tr)
        assert fn.read_text() == "a:  1\nb:  2\n"

    def test_print(self, capsys):
        yaml = YAML()
        data = yaml.map()
        data['a'] = 1
        data['b'] = 2
        yaml.dump(data, sys.stdout)
        out, err = capsys.readouterr()
        assert out == "a: 1\nb: 2\n"


class TestRead:
    def test_multi_load(self):
        # make sure reader, scanner, parser get reset
        yaml = YAML()
        yaml.load('a: 1')
        yaml.load('a: 1')  # did not work in 0.15.4


class TestDuplSet:
    def test_dupl_set_00(self):
        # round-trip-loader should except
        yaml = YAML()
        with pytest.raises(DuplicateKeyError):
            yaml.load(textwrap.dedent("""\
            !!set
            ? a
            ? b
            ? c
            ? a
            """))


class TestDumpLoadUnicode:
    # test triggered by SamH on stackoverflow (https://stackoverflow.com/q/45281596/1307905)
    # and answer by randomir (https://stackoverflow.com/a/45281922/1307905)
    def test_write_unicode(self, tmpdir):
        yaml = YAML()
        text_dict = {"text": u"HELLO_WORLD©"}
        file_name = str(tmpdir) + '/tstFile.yaml'
        yaml.dump(text_dict, open(file_name, 'w'))
        assert open(file_name, 'rb').read().decode('utf-8') == u'text: HELLO_WORLD©\n'

    def test_read_unicode(self, tmpdir):
        yaml = YAML()
        file_name = str(tmpdir) + '/tstFile.yaml'
        with open(file_name, 'wb') as fp:
            fp.write(u'text: HELLO_WORLD©\n'.encode('utf-8'))
        text_dict = yaml.load(open(file_name, 'r'))
        assert text_dict["text"] == u"HELLO_WORLD©"


class TestFlowStyle:
    def test_flow_style(self, capsys):
        # https://stackoverflow.com/questions/45791712/
        yaml = YAML()
        yaml.default_flow_style = None
        data = yaml.map()
        data['b'] = 1
        data['a'] = [[1, 2], [3, 4]]
        yaml.dump(data, sys.stdout)
        out, err = capsys.readouterr()
        assert out == "b: 1\na:\n- [1, 2]\n- [3, 4]\n"



# coding: utf-8

"""
collections.OrderedDict is a new class not supported by PyYAML (issue 83 by Frazer McLean)

This is now so integrated in Python that it can be mapped to !!omap

"""

import pytest    # NOQA
import ruamel.yaml   # NOQA
from collections import OrderedDict


from roundtrip import round_trip, dedent, round_trip_load, round_trip_dump  # NOQA


class TestOrderedDict:
    def test_ordereddict(self):
        assert ruamel.yaml.dump(OrderedDict()) == '!!omap []\n'



# coding: utf-8

import platform
import pytest


@pytest.mark.skipif(platform.python_implementation() == 'Jython',
                    reason="Jython throws RepresenterError")
def test_load_cyaml():
    import ruamel.yaml
    assert ruamel.yaml.__with_libyaml__
    from ruamel.yaml.cyaml import CLoader
    ruamel.yaml.load("abc: 1", Loader=CLoader)


def test_dump_cyaml():
    import ruamel.yaml
    data = {'a': 1, 'b': 2}
    res = ruamel.yaml.dump(data, Dumper=ruamel.yaml.cyaml.CSafeDumper,
                           default_flow_style=False, allow_unicode=True)
    assert res == "a: 1\nb: 2\n"



# coding: utf-8

import re
import pytest   # NOQA
import ruamel.yaml

from roundtrip import round_trip, dedent, round_trip_load, round_trip_dump  # NOQA


# from PyYAML docs
class Dice(tuple):
    def __new__(cls, a, b):
        return tuple.__new__(cls, [a, b])

    def __repr__(self):
        return "Dice(%s,%s)" % self


def dice_constructor(loader, node):
    value = loader.construct_scalar(node)
    a, b = map(int, value.split('d'))
    return Dice(a, b)


def dice_representer(dumper, data):
    return dumper.represent_scalar(u'!dice', u'{}d{}'.format(*data))


def test_dice_constructor():
    ruamel.yaml.add_constructor(u'!dice', dice_constructor)
    data = ruamel.yaml.load('initial hit points: !dice 8d4', Loader=ruamel.yaml.Loader)
    assert str(data) == "{'initial hit points': Dice(8,4)}"


def test_dice_constructor_with_loader():
    ruamel.yaml.add_constructor(u'!dice', dice_constructor, Loader=ruamel.yaml.Loader)
    data = ruamel.yaml.load('initial hit points: !dice 8d4', Loader=ruamel.yaml.Loader)
    assert str(data) == "{'initial hit points': Dice(8,4)}"


def test_dice_representer():
    ruamel.yaml.add_representer(Dice, dice_representer)
    # ruamel.yaml 0.15.8+ no longer forces quotes tagged scalars
    assert ruamel.yaml.dump(dict(gold=Dice(10, 6)), default_flow_style=False) == \
        "gold: !dice 10d6\n"


def test_dice_implicit_resolver():
    pattern = re.compile(r'^\d+d\d+$')
    ruamel.yaml.add_implicit_resolver(u'!dice', pattern)
    assert ruamel.yaml.dump(dict(treasure=Dice(10, 20)), default_flow_style=False) == \
        'treasure: 10d20\n'
    assert ruamel.yaml.load('damage: 5d10', Loader=ruamel.yaml.Loader) == \
        dict(damage=Dice(5, 10))


class Obj1(dict):
    def __init__(self, suffix):
        self._suffix = suffix
        self._node = None

    def add_node(self, n):
        self._node = n

    def __repr__(self):
        return 'Obj1(%s->%s)' % (self._suffix, self.items())

    def dump(self):
        return repr(self._node)


class YAMLObj1(object):
    yaml_tag = u'!obj:'

    @classmethod
    def from_yaml(cls, loader, suffix, node):
        obj1 = Obj1(suffix)
        if isinstance(node, ruamel.yaml.MappingNode):
            obj1.add_node(loader.construct_mapping(node))
        else:
            raise NotImplementedError
        return obj1

    @classmethod
    def to_yaml(cls, dumper, data):
        return dumper.represent_scalar(cls.yaml_tag + data._suffix, data.dump())


def test_yaml_obj():
    ruamel.yaml.add_representer(Obj1, YAMLObj1.to_yaml)
    ruamel.yaml.add_multi_constructor(YAMLObj1.yaml_tag, YAMLObj1.from_yaml)
    x = ruamel.yaml.load('!obj:x.2\na: 1', Loader=ruamel.yaml.Loader)
    print(x)
    assert ruamel.yaml.dump(x) == '''!obj:x.2 "{'a': 1}"\n'''


def test_yaml_obj_with_loader_and_dumper():
    ruamel.yaml.add_representer(Obj1, YAMLObj1.to_yaml, Dumper=ruamel.yaml.Dumper)
    ruamel.yaml.add_multi_constructor(YAMLObj1.yaml_tag, YAMLObj1.from_yaml,
                                      Loader=ruamel.yaml.Loader)
    x = ruamel.yaml.load('!obj:x.2\na: 1', Loader=ruamel.yaml.Loader)
    # x = ruamel.yaml.load('!obj:x.2\na: 1')
    print(x)
    assert ruamel.yaml.dump(x) == '''!obj:x.2 "{'a': 1}"\n'''


# ToDo use nullege to search add_multi_representer and add_path_resolver
# and add some test code

# Issue 127 reported by Tommy Wang

class Ref(ruamel.yaml.YAMLObject):
    yaml_constructor = ruamel.yaml.RoundTripConstructor
    yaml_representer = ruamel.yaml.RoundTripRepresenter
    yaml_tag = u'!Ref'

    def __init__(self, logical_id):
        self.logical_id = logical_id

    @classmethod
    def from_yaml(cls, loader, node):
        return cls(loader.construct_scalar(node))

    @classmethod
    def to_yaml(cls, dumper, data):
        if isinstance(data.logical_id, ruamel.yaml.scalarstring.ScalarString):
            style = data.logical_id.style   # ruamel.yaml>0.15.8
        else:
            style = None
        return dumper.represent_scalar(cls.yaml_tag, data.logical_id, style=style)


def test_issue_127():
    document = dedent('''\
    AList:
      - !Ref One
      - !Ref 'Two'
      - !Ref
        Two and a half
    BList: [!Ref Three, !Ref "Four"]
    CList:
      - Five Six
      - 'Seven Eight'
    ''')
    data = ruamel.yaml.round_trip_load(document, preserve_quotes=True)
    assert ruamel.yaml.round_trip_dump(data, indent=4, block_seq_indent=2) == \
        document.replace('\n    Two and', ' Two and')



from __future__ import absolute_import
from __future__ import print_function

import ruamel.yaml as yaml
import codecs
import tempfile
import os
import os.path
from ruamel.yaml.compat import PY2, PY3, StringIO, BytesIO

if PY2:
    def _unicode_open(file, encoding, errors='strict'):
        info = codecs.lookup(encoding)
        if isinstance(info, tuple):
            reader = info[2]
            writer = info[3]
        else:
            reader = info.streamreader
            writer = info.streamwriter
        srw = codecs.StreamReaderWriter(file, reader, writer, errors)
        srw.encoding = encoding
        return srw

if PY3:
    def test_unicode_input(unicode_filename, verbose=False):
        with open(unicode_filename, 'rb') as fp:
            data = fp.read().decode('utf-8')
        value = ' '.join(data.split())
        output = yaml.load(data)
        assert output == value, (output, value)
        output = yaml.load(StringIO(data))
        assert output == value, (output, value)
        for input in [data.encode('utf-8'),
                      codecs.BOM_UTF8 + data.encode('utf-8'),
                      codecs.BOM_UTF16_BE + data.encode('utf-16-be'),
                      codecs.BOM_UTF16_LE + data.encode('utf-16-le')]:
            if verbose:
                print("INPUT:", repr(input[:10]), "...")
            output = yaml.load(input)
            assert output == value, (output, value)
            output = yaml.load(BytesIO(input))
            assert output == value, (output, value)
else:
    def test_unicode_input(unicode_filename, verbose=False):
        with open(unicode_filename, 'rb') as fp:
            data = fp.read().decode('utf-8')
        value = ' '.join(data.split())
        output = yaml.load(_unicode_open(StringIO(data.encode('utf-8')), 'utf-8'))
        assert output == value, (output, value)
        for input in [data, data.encode('utf-8'),
                      codecs.BOM_UTF8 + data.encode('utf-8'),
                      codecs.BOM_UTF16_BE + data.encode('utf-16-be'),
                      codecs.BOM_UTF16_LE + data.encode('utf-16-le')]:
            if verbose:
                print("INPUT:", repr(input[:10]), "...")
            output = yaml.load(input)
            assert output == value, (output, value)
            output = yaml.load(StringIO(input))
            assert output == value, (output, value)

test_unicode_input.unittest = ['.unicode']


def test_unicode_input_errors(unicode_filename, verbose=False):
    with open(unicode_filename, 'rb') as fp:
        data = fp.read().decode('utf-8')
    for input in [data.encode('latin1', 'ignore'),
                  data.encode('utf-16-be'), data.encode('utf-16-le'),
                  codecs.BOM_UTF8 + data.encode('utf-16-be'),
                  codecs.BOM_UTF16_BE + data.encode('utf-16-le'),
                  codecs.BOM_UTF16_LE + data.encode('utf-8') + b'!']:
        try:
            yaml.load(input)
        except yaml.YAMLError as exc:
            if verbose:
                print(exc)
        else:
            raise AssertionError("expected an exception")
        try:
            yaml.load(BytesIO(input) if PY3 else StringIO(input))
        except yaml.YAMLError as exc:
            if verbose:
                print(exc)
        else:
            raise AssertionError("expected an exception")


test_unicode_input_errors.unittest = ['.unicode']

if PY3:
    def test_unicode_output(unicode_filename, verbose=False):
        with open(unicode_filename, 'rb') as fp:
            data = fp.read().decode('utf-8')
        value = ' '.join(data.split())
        for allow_unicode in [False, True]:
            data1 = yaml.dump(value, allow_unicode=allow_unicode)
            for encoding in [None, 'utf-8', 'utf-16-be', 'utf-16-le']:
                stream = StringIO()
                yaml.dump(value, stream, encoding=encoding, allow_unicode=allow_unicode)
                data2 = stream.getvalue()
                data3 = yaml.dump(value, encoding=encoding, allow_unicode=allow_unicode)
                if encoding is not None:
                    assert isinstance(data3, bytes)
                    data3 = data3.decode(encoding)
                stream = BytesIO()
                if encoding is None:
                    try:
                        yaml.dump(value, stream, encoding=encoding,
                                  allow_unicode=allow_unicode)
                    except TypeError as exc:
                        if verbose:
                            print(exc)
                        data4 = None
                    else:
                        raise AssertionError("expected an exception")
                else:
                    yaml.dump(value, stream, encoding=encoding, allow_unicode=allow_unicode)
                    data4 = stream.getvalue()
                    if verbose:
                        print("BYTES:", data4[:50])
                    data4 = data4.decode(encoding)
                for copy in [data1, data2, data3, data4]:
                    if copy is None:
                        continue
                    assert isinstance(copy, str)
                    if allow_unicode:
                        try:
                            copy[4:].encode('ascii')
                        except UnicodeEncodeError as exc:
                            if verbose:
                                print(exc)
                        else:
                            raise AssertionError("expected an exception")
                    else:
                        copy[4:].encode('ascii')
                assert isinstance(data1, str), (type(data1), encoding)
                assert isinstance(data2, str), (type(data2), encoding)
else:
    def test_unicode_output(unicode_filename, verbose=False):
        with open(unicode_filename, 'rb') as fp:
            data = fp.read().decode('utf-8')
        value = ' '.join(data.split())
        for allow_unicode in [False, True]:
            data1 = yaml.dump(value, allow_unicode=allow_unicode)
            for encoding in [None, 'utf-8', 'utf-16-be', 'utf-16-le']:
                stream = StringIO()
                yaml.dump(value, _unicode_open(stream, 'utf-8'), encoding=encoding,
                          allow_unicode=allow_unicode)
                data2 = stream.getvalue()
                data3 = yaml.dump(value, encoding=encoding, allow_unicode=allow_unicode)
                stream = StringIO()
                yaml.dump(value, stream, encoding=encoding, allow_unicode=allow_unicode)
                data4 = stream.getvalue()
                for copy in [data1, data2, data3, data4]:
                    if allow_unicode:
                        try:
                            copy[4:].encode('ascii')
                        except (UnicodeDecodeError, UnicodeEncodeError) as exc:
                            if verbose:
                                print(exc)
                        else:
                            raise AssertionError("expected an exception")
                    else:
                        copy[4:].encode('ascii')
                assert isinstance(data1, str), (type(data1), encoding)
                data1.decode('utf-8')
                assert isinstance(data2, str), (type(data2), encoding)
                data2.decode('utf-8')
                if encoding is None:
                    assert isinstance(data3, unicode), (type(data3), encoding)  # NOQA
                    assert isinstance(data4, unicode), (type(data4), encoding)  # NOQA
                else:
                    assert isinstance(data3, str), (type(data3), encoding)
                    data3.decode(encoding)
                    assert isinstance(data4, str), (type(data4), encoding)
                    data4.decode(encoding)


test_unicode_output.unittest = ['.unicode']


def test_file_output(unicode_filename, verbose=False):
    with open(unicode_filename, 'rb') as fp:
        data = fp.read().decode('utf-8')
    handle, filename = tempfile.mkstemp()
    os.close(handle)
    try:
        stream = StringIO()
        yaml.dump(data, stream, allow_unicode=True)
        data1 = stream.getvalue()
        if PY3:
            stream = BytesIO()
            yaml.dump(data, stream, encoding='utf-16-le', allow_unicode=True)
            data2 = stream.getvalue().decode('utf-16-le')[1:]
            with open(filename, 'w', encoding='utf-16-le') as stream:
                yaml.dump(data, stream, allow_unicode=True)
            with open(filename, 'r', encoding='utf-16-le') as fp0:
                data3 = fp0.read()
            with open(filename, 'wb') as stream:
                yaml.dump(data, stream, encoding='utf-8', allow_unicode=True)
            with open(filename, 'r', encoding='utf-8') as fp0:
                data4 = fp0.read()
        else:
            with open(filename, 'wb') as stream:
                yaml.dump(data, stream, allow_unicode=True)
            with open(filename, 'rb') as fp0:
                data2 = fp0.read()
            with open(filename, 'wb') as stream:
                yaml.dump(data, stream, encoding='utf-16-le',
                          allow_unicode=True)
            with open(filename, 'rb') as fp0:
                data3 = fp0.read().decode('utf-16-le')[1:].encode('utf-8')
            stream = _unicode_open(open(filename, 'wb'), 'utf-8')
            yaml.dump(data, stream, allow_unicode=True)
            stream.close()
            with open(filename, 'rb') as fp0:
                data4 = fp0.read()
        assert data1 == data2, (data1, data2)
        assert data1 == data3, (data1, data3)
        assert data1 == data4, (data1, data4)
    finally:
        if os.path.exists(filename):
            os.unlink(filename)


test_file_output.unittest = ['.unicode']


def test_unicode_transfer(unicode_filename, verbose=False):
    with open(unicode_filename, 'rb') as fp:
        data = fp.read().decode('utf-8')
    for encoding in [None, 'utf-8', 'utf-16-be', 'utf-16-le']:
        input = data
        if PY3:
            if encoding is not None:
                input = ('\ufeff' + input).encode(encoding)
            output1 = yaml.emit(yaml.parse(input), allow_unicode=True)
            if encoding is None:
                stream = StringIO()
            else:
                stream = BytesIO()
            yaml.emit(yaml.parse(input), stream, allow_unicode=True)
            output2 = stream.getvalue()
            assert isinstance(output1, str), (type(output1), encoding)
            if encoding is None:
                assert isinstance(output2, str), (type(output1), encoding)
            else:
                assert isinstance(output2, bytes), (type(output1), encoding)
                output2.decode(encoding)
        else:
            if encoding is not None:
                input = (u'\ufeff' + input).encode(encoding)
            output1 = yaml.emit(yaml.parse(input), allow_unicode=True)
            stream = StringIO()
            yaml.emit(yaml.parse(input), _unicode_open(stream, 'utf-8'), allow_unicode=True)
            output2 = stream.getvalue()
            if encoding is None:
                assert isinstance(output1, unicode), (type(output1), encoding)  # NOQA
            else:
                assert isinstance(output1, str), (type(output1), encoding)
                output1.decode(encoding)
            assert isinstance(output2, str), (type(output2), encoding)
            output2.decode('utf-8')


test_unicode_transfer.unittest = ['.unicode']

if __name__ == '__main__':
    import test_appliance
    test_appliance.run(globals())



# coding: utf-8

from __future__ import print_function

"""
various test cases for string scalars in YAML files
'|' for preserved newlines
'>' for folded (newlines become spaces)

and the chomping modifiers:
'-' for stripping: final line break and any trailing empty lines are excluded
'+' for keeping: final line break and empty lines are preserved
''  for clipping: final line break preserved, empty lines at end not
    included in content (no modifier)

"""

import pytest
import platform
import ruamel

# from ruamel.yaml.compat import ordereddict
from roundtrip import round_trip, dedent, round_trip_load, round_trip_dump  # NOQA


class TestPreservedScalarString:
    def test_basic_string(self):
        round_trip("""
        a: abcdefg
        """, )

    def test_quoted_integer_string(self):
        round_trip("""
        a: '12345'
        """)

    @pytest.mark.skipif(platform.python_implementation() == 'Jython',
                        reason="Jython throws RepresenterError")
    def test_preserve_string(self):
        round_trip("""
            a: |
              abc
              def
            """, intermediate=dict(a='abc\ndef\n'))

    @pytest.mark.skipif(platform.python_implementation() == 'Jython',
                        reason="Jython throws RepresenterError")
    def test_preserve_string_strip(self):
        s = """
            a: |-
              abc
              def

            """
        round_trip(s, intermediate=dict(a='abc\ndef'))

    @pytest.mark.skipif(platform.python_implementation() == 'Jython',
                        reason="Jython throws RepresenterError")
    def test_preserve_string_keep(self):
            # with pytest.raises(AssertionError) as excinfo:
            round_trip("""
            a: |+
              ghi
              jkl


            b: x
            """, intermediate=dict(a='ghi\njkl\n\n\n', b='x'))

    @pytest.mark.skipif(platform.python_implementation() == 'Jython',
                        reason="Jython throws RepresenterError")
    def test_preserve_string_keep_at_end(self):
        # at EOF you have to specify the ... to get proper "closure"
        # of the multiline scalar
        round_trip("""
            a: |+
              ghi
              jkl

            ...
            """, intermediate=dict(a='ghi\njkl\n\n'))

    def test_fold_string(self):
        with pytest.raises(AssertionError) as excinfo:  # NOQA
            round_trip("""
            a: >
              abc
              def

            """, intermediate=dict(a='abc def\n'))

    def test_fold_string_strip(self):
        with pytest.raises(AssertionError) as excinfo:  # NOQA
            round_trip("""
            a: >-
              abc
              def

            """, intermediate=dict(a='abc def'))

    def test_fold_string_keep(self):
        with pytest.raises(AssertionError) as excinfo:  # NOQA
            round_trip("""
            a: >+
              abc
              def

            """, intermediate=dict(a='abc def\n\n'))


class TestQuotedScalarString:
    def test_single_quoted_string(self):
        round_trip("""
        a: 'abc'
        """, preserve_quotes=True)

    def test_double_quoted_string(self):
        round_trip("""
        a: "abc"
        """, preserve_quotes=True)

    def test_non_preserved_double_quoted_string(self):
        round_trip("""
        a: "abc"
        """, outp="""
        a: abc
        """)


class TestReplace:
    """inspired by issue 110 from sandres23"""
    def test_replace_preserved_scalar_string(self):
        s = dedent("""\
        foo: |
          foo
          foo
          bar
          foo
        """)
        data = round_trip_load(s, preserve_quotes=True)
        so = data['foo'].replace('foo', 'bar', 2)
        assert isinstance(so, ruamel.yaml.scalarstring.PreservedScalarString)
        assert so == dedent("""
        bar
        bar
        bar
        foo
        """)

    def test_replace_double_quoted_scalar_string(self):
        s = dedent("""\
        foo: "foo foo bar foo"
        """)
        data = round_trip_load(s, preserve_quotes=True)
        so = data['foo'].replace('foo', 'bar', 2)
        assert isinstance(so, ruamel.yaml.scalarstring.DoubleQuotedScalarString)
        assert so == 'bar bar bar foo'




from __future__ import print_function

"""
helper routines for testing round trip of commented YAML data
"""
import sys
import textwrap

import ruamel.yaml
from ruamel.yaml.compat import StringIO, BytesIO  # NOQA


def dedent(data):
    try:
        position_of_first_newline = data.index('\n')
        for idx in range(position_of_first_newline):
            if not data[idx].isspace():
                raise ValueError
    except ValueError:
        pass
    else:
        data = data[position_of_first_newline + 1:]
    return textwrap.dedent(data)


def round_trip_load(inp, preserve_quotes=None, version=None):
    dinp = dedent(inp)
    return ruamel.yaml.load(
        dinp,
        Loader=ruamel.yaml.RoundTripLoader,
        preserve_quotes=preserve_quotes,
        version=version,
    )


def round_trip_load_all(inp, preserve_quotes=None, version=None):
    dinp = dedent(inp)
    return ruamel.yaml.load_all(
        dinp,
        Loader=ruamel.yaml.RoundTripLoader,
        preserve_quotes=preserve_quotes,
        version=version,
    )


def round_trip_dump(data, stream=None,
                    indent=None, block_seq_indent=None, top_level_colon_align=None,
                    prefix_colon=None, explicit_start=None, explicit_end=None, version=None):
    return ruamel.yaml.round_trip_dump(data, stream=stream,
                                       indent=indent, block_seq_indent=block_seq_indent,
                                       top_level_colon_align=top_level_colon_align,
                                       prefix_colon=prefix_colon,
                                       explicit_start=explicit_start,
                                       explicit_end=explicit_end,
                                       version=version)


def diff(inp, outp, file_name='stdin'):
    import difflib
    inl = inp.splitlines(True)  # True for keepends
    outl = outp.splitlines(True)
    diff = difflib.unified_diff(inl, outl, file_name, 'round trip YAML')
    # 2.6 difflib has trailing space on filename lines %-)
    strip_trailing_space = sys.version_info < (2, 7)
    for line in diff:
        if strip_trailing_space and line[:4] in ['--- ', '+++ ']:
            line = line.rstrip() + '\n'
        sys.stdout.write(line)


def round_trip(inp, outp=None, extra=None, intermediate=None, indent=None,
               block_seq_indent=None, top_level_colon_align=None, prefix_colon=None,
               preserve_quotes=None,
               explicit_start=None, explicit_end=None,
               version=None):
    """
    inp:    input string to parse
    outp:   expected output (equals input if not specified)
    """
    if outp is None:
        outp = inp
    doutp = dedent(outp)
    if extra is not None:
        doutp += extra
    data = round_trip_load(inp, preserve_quotes=preserve_quotes)
    if intermediate is not None:
        if isinstance(intermediate, dict):
            for k, v in intermediate.items():
                if data[k] != v:
                    print('{0!r} <> {1!r}'.format(data[k], v))
                    raise ValueError
    res = round_trip_dump(data, indent=indent, block_seq_indent=block_seq_indent,
                          top_level_colon_align=top_level_colon_align,
                          prefix_colon=prefix_colon,
                          explicit_start=explicit_start,
                          explicit_end=explicit_end,
                          version=version)
    if res != doutp:
        diff(doutp, res, "input string")
    print('\nroundtrip data:\n', res, sep='')
    assert res == doutp
    res = round_trip_dump(data, indent=indent, block_seq_indent=block_seq_indent,
                          top_level_colon_align=top_level_colon_align,
                          prefix_colon=prefix_colon,
                          explicit_start=explicit_start,
                          explicit_end=explicit_end,
                          version=version)
    print('roundtrip second round data:\n', res, sep='')
    assert res == doutp
    return data


class YAML(ruamel.yaml.YAML):
    """auto dedent string parameters on load"""
    def load(self, stream):
        if isinstance(stream, str):
            if stream and stream[0] == '\n':
                stream = stream[1:]
            stream = textwrap.dedent(stream)
        return ruamel.yaml.YAML.load(self, stream)

    def load_all(self, stream):
        if isinstance(stream, str):
            if stream and stream[0] == '\n':
                stream = stream[1:]
            stream = textwrap.dedent(stream)
        for d in ruamel.yaml.YAML.load_all(self, stream):
            yield d

    def dump(self, data, **kw):
        assert ('stream' in kw) ^ ('compare' in kw)
        if 'stream' in kw:
            return ruamel.yaml.YAML.dump(data, **kw)
        lkw = kw.copy()
        expected = textwrap.dedent(lkw.pop('compare'))
        unordered_lines = lkw.pop('unordered_lines', False)
        if expected and expected[0] == '\n':
            expected = expected[1:]
        lkw['stream'] = st = StringIO()
        ruamel.yaml.YAML.dump(self, data, **lkw)
        res = st.getvalue()
        print(res)
        if unordered_lines:
            res = sorted(res.splitlines())
            expected = sorted(expected.splitlines())
        assert res == expected

    def round_trip(self, stream, **kw):
        assert isinstance(stream, ruamel.yaml.compat.text_type)
        lkw = kw.copy()
        if stream and stream[0] == '\n':
            stream = stream[1:]
        stream = textwrap.dedent(stream)
        data = ruamel.yaml.YAML.load(self, stream)
        outp = lkw.pop('outp', stream)
        lkw['stream'] = st = StringIO()
        ruamel.yaml.YAML.dump(self, data, **lkw)
        res = st.getvalue()
        if res != outp:
            diff(outp, res, "input string")
        assert res == outp



# coding: utf-8

from __future__ import print_function, absolute_import, division, unicode_literals

import pytest  # NOQA

from roundtrip import round_trip, dedent, round_trip_load, round_trip_dump

# http://yaml.org/type/int.html is where underscores in integers are defined


class TestBinHexOct:
    # @pytest.mark.xfail(strict=True)
    def test_round_trip_hex_oct(self):
        round_trip("""\
        - 42
        - 0b101010
        - 0x2a
        - 0x2A
        - 0o52
        """)

    def test_calculate(self):
        # make sure type, leading zero(s) and underscore are preserved
        s = dedent("""\
        - 42
        - 0b101010
        - 0x_2a
        - 0x2A
        - 0o00_52
        """)
        x = round_trip_load(s)
        for idx, elem in enumerate(x):
            # x[idx] = type(elem)(elem - 21)
            elem -= 21
            x[idx] = elem
        for idx, elem in enumerate(x):
            # x[idx] = type(elem)(2 * elem)
            elem *= 2
            x[idx] = elem
        for idx, elem in enumerate(x):
            t = elem
            elem **= 2
            elem //= t
            x[idx] = elem
        assert round_trip_dump(x) == s

    # if a scalar int has one or more leading zeros, it is assumed that the width
    # of the int is significant, as padding with a zero doesn't make much sense
    # please note that none of this should work on YAML 1.1 as it collides with
    # the old octal representation.

    def test_leading_zero_hex_oct_bin(self):
        round_trip("""\
        - 0b0101010
        - 0b00101010
        - 0x02a
        - 0x002a
        - 0x02A
        - 0x002A
        - 0o052
        - 0o0052
        """)

    def test_leading_zero_int(self):
        round_trip("""\
        - 042
        - 0042
        """)

    def test_leading_zero_YAML_1_1(self):
        d = round_trip_load("""\
        %YAML 1.1
        ---
        - 042
        - 0o42
        """)
        assert d[0] == 0o42
        assert d[1] == '0o42'

    def test_underscore(self):
        round_trip("""\
        - 0b10000_10010010
        - 0b0_0000_1001_0010
        - 0x2_87_57_b2_
        - 0x0287_57B2
        - 0x_0_2_8_7_5_7_B_2
        - 0o2416_53662
        - 42_42_
        """)

    def test_leading_underscore(self):
        d = round_trip_load("""\
        - 0x_2_8_7_5_7_B_2
        - _42_42_
        - 42_42_
        """)
        assert d[0] == 42424242
        assert d[1] == '_42_42_'
        assert d[2] == 4242

    def test_big(self):
        # bitbucket issue 144 reported by ccatterina
        d = round_trip_load("""\
        - 2_147_483_647
        - 9_223_372_036_854_775_808
        """)
        assert d[0] == 2147483647
        assert d[1] == 9223372036854775808



# coding: utf-8

from __future__ import print_function, absolute_import, division, unicode_literals

import pytest  # NOQA

from roundtrip import round_trip, dedent, round_trip_load, round_trip_dump  # NOQA
from ruamel.yaml.error import MantissaNoDotYAML1_1Warning

# http://yaml.org/type/int.html is where underscores in integers are defined


class TestFloat:
    def test_round_trip_non_exp(self):
        data = round_trip("""\
        - 1.0
        - 1.00
        - 23.100
        - -1.0
        - -1.00
        - -23.100
        - 42.
        - -42.
        - +42.
        """)
        print(data)
        assert 0.999 < data[0] < 1.001
        assert 0.999 < data[1] < 1.001
        assert 23.099 < data[2] < 23.101
        assert 0.999 < -data[3] < 1.001
        assert 0.999 < -data[4] < 1.001
        assert 23.099 < -data[5] < 23.101
        assert 41.999 < data[6] < 42.001
        assert 41.999 < -data[7] < 42.001
        assert 41.999 < data[8] < 42.001

    def test_round_trip_zeros_0(self):
        data = round_trip("""\
        - 0.
        - +0.
        - -0.
        - 0.0
        - +0.0
        - -0.0
        - 0.00
        - +0.00
        - -0.00
        """)
        print(data)
        for d in data:
            assert -0.00001 < d < 0.00001

    # @pytest.mark.xfail(strict=True)
    def test_round_trip_zeros_1(self):
        # not sure if this should be supported, but it is
        data = round_trip("""\
        - 00.0
        - +00.0
        - -00.0
        """)
        print(data)
        for d in data:
            assert -0.00001 < d < 0.00001

    def Xtest_round_trip_non_exp_trailing_dot(self):
        data = round_trip("""\
        """)
        print(data)

    def test_round_trip_exp_00(self):
        data = round_trip("""\
        - 42e56
        - 42E56
        - 42.0E56
        - +42.0e56
        - 42.0E+056
        - +42.00e+056
        """)
        print(data)
        for d in data:
            assert 41.99e56 < d < 42.01e56

    # @pytest.mark.xfail(strict=True)
    def test_round_trip_exp_00f(self):
        data = round_trip("""\
        - 42.E56
        """)
        print(data)
        for d in data:
            assert 41.99e56 < d < 42.01e56

    def test_round_trip_exp_01(self):
        data = round_trip("""\
        - -42e56
        - -42E56
        - -42.0e56
        - -42.0E+056
        """)
        print(data)
        for d in data:
            assert -41.99e56 > d > -42.01e56

    def test_round_trip_exp_02(self):
        data = round_trip("""\
        - 42e-56
        - 42E-56
        - 42.0E-56
        - +42.0e-56
        - 42.0E-056
        - +42.0e-056
        """)
        print(data)
        for d in data:
            assert 41.99e-56 < d < 42.01e-56

    def test_round_trip_exp_03(self):
        data = round_trip("""\
        - -42e-56
        - -42E-56
        - -42.0e-56
        - -42.0E-056
        """)
        print(data)
        for d in data:
            assert -41.99e-56 > d > -42.01e-56

    def test_round_trip_exp_04(self):
        round_trip("""\
        - 1.2e+34
        - 1.23e+034
        - 1.230e+34
        - 1.023e+34
        - -1.023e+34
        - 250e6
        """)

    def test_round_trip_exp_05(self):
        data = round_trip("""\
        - 3.0517578123e-56
        - 3.0517578123E-56
        - 3.0517578123e-056
        - 3.0517578123E-056
        """)
        print(data)
        for d in data:
            assert 3.0517578122e-56 < d < 3.0517578124e-56

    def test_yaml_1_1_no_dot(self):
        with pytest.warns(MantissaNoDotYAML1_1Warning):
            round_trip_load("""\
            %YAML 1.1
            ---
            - 1e6
            """)


class TestCalculations(object):
    def test_mul_00(self):
        # issue 149 reported by jan.brezina@tul.cz
        d = round_trip_load("""\
        - 0.1
        """)
        d[0] *= -1
        x = round_trip_dump(d)
        assert x == '- -0.1\n'



# coding: utf-8

from __future__ import absolute_import, print_function, unicode_literals


import pytest  # NOQA

import ruamel.yaml

from roundtrip import round_trip, round_trip_load, round_trip_dump, dedent  # NOQA


class TestIssue61:
    def test_issue_61(self):
        s = dedent("""
        def1: &ANCHOR1
            key1: value1
        def: &ANCHOR
            <<: *ANCHOR1
            key: value
        comb:
            <<: *ANCHOR
        """)
        data = ruamel.yaml.round_trip_load(s)
        assert str(data['comb']) == str(data['def'])
        assert str(data['comb']) == "ordereddict([('key', 'value'), ('key1', 'value1')])"



# coding: utf-8

from __future__ import print_function, absolute_import, division, unicode_literals

try:
    import numpy
except:  # NOQA
    numpy = None

import ruamel.yaml


def Xtest_numpy():
    if numpy is None:
        return
    data = numpy.arange(10)
    print('data', type(data), data)

    yaml_str = ruamel.yaml.dump(data)
    datb = ruamel.yaml.load(yaml_str)
    print('datb', type(datb), datb)

    print('\nYAML', yaml_str)
    assert data == datb
